Package: maotai
Type: Package
Title: Tools for Matrix Algebra, Optimization and Inference
Version: 0.3.0
Authors@R: c(person("Kisung", "You", role = c("aut", "cre"), email = "kisung.you@outlook.com",comment=c(ORCID="0000-0002-8584-459X")))
Description: Matrix is an universal and sometimes primary object/unit in applied mathematics and statistics. We provide a number of algorithms for selected problems in optimization and statistical inference. For general exposition to the topic with focus on statistical context, see the book by Banerjee and Roy (2014, ISBN:9781420095388).
Encoding: UTF-8
License: MIT + file LICENSE
Suggests: covr, igraph, testthat (>= 3.0.0)
Imports: Matrix, Rcpp, Rdpack, RSpectra, Rtsne, RANN, cluster, labdsv,
        stats, utils, fastcluster, dbscan, pracma, gsignal
LinkingTo: Rcpp, RcppArmadillo, RcppDist
RdMacros: Rdpack
RoxygenNote: 7.3.3
URL: https://github.com/kisungyou/maotai
BugReports: https://github.com/kisungyou/maotai/issues
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-01-12 17:35:25 UTC; kyou
Author: Kisung You [aut, cre] (ORCID: <https://orcid.org/0000-0002-8584-459X>)
Maintainer: Kisung You <kisung.you@outlook.com>
Repository: CRAN
Date/Publication: 2026-01-13 06:10:23 UTC
