# Generated by roxygen2: do not edit by hand

export(LEiDA)
export(WLbarycenter)
export(WLmedian)
export(WLpdist)
export(bmds)
export(boot.mblock)
export(boot.stationary)
export(bwproj)
export(cayleymenger)
export(checkdist)
export(checkmetric)
export(cmds)
export(cov2corr)
export(cov2pcorr)
export(dpmeans)
export(ecdfdist)
export(ecdfdist2)
export(ecdfdistS)
export(epmeans)
export(kmeanspp)
export(lyapunov)
export(matderiv)
export(metricdepth)
export(mmd2test)
export(movMF_convert)
export(movMF_info)
export(movMF_reduce_greedy)
export(movMF_reduce_partitional)
export(nef)
export(nem)
export(pdeterminant)
export(rotationS2)
export(shortestpath)
export(sylvester)
export(trio)
export(tsne)
export(weiszfeld)
import(Rdpack)
importFrom(Matrix,rankMatrix)
importFrom(RANN,nn2)
importFrom(RSpectra,eigs)
importFrom(Rcpp,evalCpp)
importFrom(Rtsne,Rtsne)
importFrom(cluster,agnes)
importFrom(cluster,pam)
importFrom(cluster,silhouette)
importFrom(dbscan,dbscan)
importFrom(fastcluster,hclust)
importFrom(gsignal,butter)
importFrom(gsignal,filtfilt)
importFrom(gsignal,hilbert)
importFrom(pracma,cross)
importFrom(pracma,detrend)
importFrom(stats,as.dist)
importFrom(stats,cov)
importFrom(stats,dist)
importFrom(stats,ecdf)
importFrom(stats,knots)
importFrom(stats,lm)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,rgeom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(utils,packageVersion)
useDynLib(maotai)
