
<!-- README.md is generated from README.Rmd. Please edit that file -->

# Tools for Matrix Algebra, Optimization and Inference Problems

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/maotai)](https://CRAN.R-project.org/package=maotai)
[![R-CMD-check](https://github.com/kisungyou/maotai/workflows/R-CMD-check/badge.svg)](https://github.com/kisungyou/maotai/actions)
[![Codecov test
coverage](https://codecov.io/gh/kisungyou/maotai/branch/master/graph/badge.svg)](https://app.codecov.io/gh/kisungyou/maotai?branch=master)
[![metacran
downloads](https://cranlogs.r-pkg.org/badges/grand-total/maotai)](https://cran.r-project.org/package=maotai)
<!-- badges: end -->

**maotai** is an R package whose name is an acronym for **M**atrix
**A**lgebra, **O**p**T**imization, **A**nd **I**nference problems -
though I can’t deny motivation from one of [my father’s
favorite](https://en.wikipedia.org/wiki/Maotai) for the namesake. More
detailed introduction will be added later.

## Installation

Install the latest release from
[CRAN](https://CRAN.R-project.org/package=maotai) with:

``` r
install.packages("maotai")
```

or the up-to-date development version from github:

``` r
# install.packages("devtools")
devtools::install_github("kisungyou/maotai")
```
